unit ufmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ufrWaveSetup, TeEngine, Series, ExtCtrls, TeeProcs,
  Chart, uSignalIO, ufrAccumulation, uGUIUtils, ufrGraph, TeeBezie;

const
  WM_VOLUME = WM_USER;

type
  TForm1 = class(TScaledForm)
    frWaveSetup1: TfrWaveSetup;
    Chart1: TChart;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    btStart: TButton;
    cbMono: TCheckBox;
    ScrollBar1: TScrollBar;
    Label1: TLabel;
    frAccumulation1: TfrAccumulation;
    ScrollBar2: TScrollBar;
    lbVol: TLabel;
    cbFFactor: TComboBox;
    Label2: TLabel;
    edAFactor: TEdit;
    Label3: TLabel;
    Series3: TLineSeries;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btStartClick(Sender: TObject);
    procedure Chart1AfterDraw(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure ScrollBar2Change(Sender: TObject);
    procedure edAFactorChange(Sender: TObject);
  private
    { Dclarations prives }
    FSignalIn: TSignalIn;
    FSignalOut: TSignalOut;
    FAFactor: Double;
    procedure LoadIni;
    procedure SaveIni;
    procedure ShowData;
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  IniFiles, uInterfaces, uChartSettings;

const
  csSection = 'Main';

procedure TForm1.LoadIni;
var
  aFile: TIniFile;
  i: Integer;
  aIniIO: IIniIO;
begin
  aFile:= TIniFile.Create(ChangeFileExt(Paramstr(0),'.ini'));
  edAFactor.Text:= aFile.ReadString(csSection, 'AFactor', edAFactor.Text);
  edAFactorChange(nil);
  ScrollBar1.Position:= aFile.ReadInteger(csSection, 'FPosition', ScrollBar1.Position);
  cbFFactor.ItemIndex:= aFile.ReadInteger(csSection, 'FFactor', cbFFactor.ItemIndex);
  ScrollBar1Change(nil);
  for i:= 0 to ComponentCount - 1 do begin
    Components[i].GetInterface(IIniIO, aIniIO);
    if Assigned(aIniIO) then
      aIniIO.LoadIni(aFile);
  end;
  aFile.Free;
end;

procedure TForm1.SaveIni;
var
  aFile: TIniFile;
  i: Integer;
  aIniIO: IIniIO;
begin
  aFile:= TIniFile.Create(ChangeFileExt(Paramstr(0),'.ini'));
  aFile.WriteString(csSection, 'AFactor', edAFactor.Text);
  aFile.WriteInteger(csSection, 'FPosition', ScrollBar1.Position);
  aFile.WriteInteger(csSection, 'FFactor', cbFFactor.ItemIndex);
  for i:= 0 to ComponentCount - 1 do begin
    Components[i].GetInterface(IIniIO, aIniIO);
    if Assigned(aIniIO) then
      aIniIO.SaveIni(aFile);
  end;  
  aFile.Free;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FAFactor:= 1;
  InitChart(Chart1);
  FSignalIn:= TSignalIn.Create(frWaveSetup1);
  FSignalOut:= TSignalOut.Create(frWaveSetup1);
  FSignalIn.OnDataReady:= ShowData;
  frAccumulation1.SetAccMode:= FSignalIn.SetAccMode;
  cbFFactor.Clear;
  cbFFactor.Items.AddObject('1', TObject(1));
  cbFFactor.Items.AddObject('10', TObject(10));
  cbFFactor.Items.AddObject('100', TObject(100));
  cbFFactor.ItemIndex:= 0;
  LoadIni;
  ScrollBar2.Position:= ScrollBar2.Max - FSignalOut.Volume;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  SaveIni;
  FSignalIn.Free;
  FSignalOut.Free;
end;

procedure TForm1.ShowData;
var
  i, j: Integer;
  aD: array [0..1] of Extended;
  aSeries: array[0..1] of TChartSeries;
begin
  frAccumulation1.lbAccCount.Caption:= 'N= ' + IntToStr(FSignalIn.AccCount);
  aSeries[0]:= Series1;
  aSeries[1]:= Series2;
  Series1.Clear;
  Series2.Clear;
  Series3.Clear;
  for i:= 0 to FSignalIn.Channels - 1 do begin
    aD[i]:= 0;
    for j:= 0 to High(FSignalIn.Data[i]) do
      aD[i]:= aD[i] + FSignalIn.Data[i,j];
    aD[i]:= aD[i] / Length(FSignalIn.Data[i]);
    //if FSignalIn.AccCount = 1 then
    //  aD[i]:= Round(aD[i]);
  end;
  for i:= 0 to FSignalIn.Channels - 1 do begin
    //aD[i]:= 0;
    for j:= 0 to High(FSignalIn.Data[i]) do
      aSeries[i].AddXY(j, FAFactor*(FSignalIn.Data[i,j] - aD[i]));
  end;
  for j:= 0 to High(FSignalIn.Data[0]) do
    Series3.AddXY(j, FAFactor*(FSignalIn.Data[0,j] - aD[0]));
end;

procedure TForm1.btStartClick(Sender: TObject);
begin
  if FSignalIn.Running then begin
    FSignalIn.Stop;
    FSignalOut.Stop;
    btStart.Caption:= 'Start';
    frWaveSetup1.Enabled:= True;
    cbMono.Enabled:= True;
  end else begin
    ScrollBar1Change(Sender);
    if cbMono.Checked then
      FSignalIn.Channels:= 1
    else
      FSignalIn.Channels:= 2;
    if cbMono.Checked then
      FSignalOut.Channels:= 1
    else
      FSignalOut.Channels:= 2;
    cbMono.Enabled:= False;
    frWaveSetup1.Enabled:= False;
    FSignalOut.Start;
    FSignalIn.Start;
    btStart.Caption:= 'Stop';
  end
end;

procedure TForm1.Chart1AfterDraw(Sender: TObject);
begin
  Application.ProcessMessages;
  FSignalIn.CanSend;
end;

procedure TForm1.ScrollBar1Change(Sender: TObject);
var
  N: Integer;
begin
  N:= ScrollBar1.Position*Integer(cbFFactor.Items.Objects[cbFFactor.ItemIndex]);
  Label1.Caption:= IntToStr(N) + ': ' +
  FloatToStrF(frWaveSetup1.Sampling / N, ffFixed, 6, 2)
  + ' Hz';
  if FSignalIn.Running then begin
    FSignalIn.Stop;
    FSignalOut.Stop;
    FSignalIn.Count:= N*2;
    FSignalOut.SetCount(N);
    FSignalOut.Start;
    FSignalIn.Start;
  end else begin
    FSignalIn.Count:= N*2;
    FSignalOut.SetCount(N);
  end;
end;

procedure TForm1.ScrollBar2Change(Sender: TObject);
var
  aVolume: Integer;
begin
  aVolume:= ScrollBar2.Max - ScrollBar2.Position;
  FSignalOut.Volume:= aVolume;
  lbVol.Caption:= IntToStr(aVolume);
end;

procedure TForm1.edAFactorChange(Sender: TObject);
var
  aVal: Single;
  aCode: Integer;
  aSt: String;
begin
  aSt:= Trim(edAFactor.Text);
  Val(aSt, aVal, aCode);
  if aCode = 0 then begin
    edAFactor.Font.Color:= clWindowText;
    FAFactor:= aVal;
  end else
    edAFactor.Font.Color:= clRed;
end;

end.


