unit ufrWaveSetup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, IniFiles, uInterfaces;

type
  TfrWaveSetup = class(TFrame, IIniIO, IWaveSetup)
    cbSampling: TComboBox;
    lbSampling: TLabel;
    lbInput: TLabel;
    cbDeviceIn: TComboBox;
    lbOutput: TLabel;
    cbDeviceOut: TComboBox;
    cbBits: TComboBox;
    lbBits: TLabel;
  private
    { Private declarations }
  protected
    procedure SetEnabled(Value: Boolean); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure LoadIni(aFile: TIniFile); stdcall;
    procedure SaveIni(aFile: TIniFile); stdcall;
    function Bits: Word; stdcall;
    function Sampling: LongWord; stdcall;
    function Input: Integer; stdcall;
    function Output: Integer; stdcall;
  end;

implementation

{$R *.DFM}

uses
  MMSystem;

const
  csSection = 'WaveSetup';

constructor TfrWaveSetup.Create(AOwner: TComponent);
var
  i: Integer;
  aCaps: WAVEINCAPS;
begin
  inherited Create(AOwner);
  cbBits.ItemIndex:= 1;
  for i:= 0 to cbBits.Items.Count - 1 do
    cbBits.Items.Objects[i]:= TObject(StrToInt(cbBits.Items[i]));
  cbSampling.ItemIndex:= 1;
  for i:= 0 to cbSampling.Items.Count - 1 do
    cbSampling.Items.Objects[i]:= TObject(StrToInt(cbSampling.Items[i]));
  cbDeviceIn.Items.Clear;
  for i:= 0 to waveInGetNumDevs - 1 do begin
    waveInGetDevCaps(i, @aCaps, SizeOf(aCaps));
    cbDeviceIn.Items.Add(aCaps.szPname);
  end;
  if cbDeviceIn.Items.Count > 0 then
    cbDeviceIn.ItemIndex:= 0;
  cbDeviceOut.Items.Clear;
  for i:= 0 to waveOutGetNumDevs - 1 do begin
    waveOutGetDevCaps(i, @aCaps, SizeOf(aCaps));
    cbDeviceOut.Items.Add(aCaps.szPname);
  end;
  if cbDeviceOut.Items.Count > 0 then
    cbDeviceOut.ItemIndex:= 0;
end;

procedure TfrWaveSetup.LoadIni(aFile: TIniFile);
var
  aInt, i: Integer;
begin
  aInt:= aFile.ReadInteger(csSection, 'Input', cbDeviceIn.ItemIndex);
  if aInt < cbDeviceIn.Items.Count then
    cbDeviceIn.ItemIndex:= aInt;
  aInt:= aFile.ReadInteger(csSection, 'Output', cbDeviceOut.ItemIndex);
  if aInt < cbDeviceOut.Items.Count then
    cbDeviceOut.ItemIndex:= aInt;
  aInt:= aFile.ReadInteger(csSection, 'Bits',
         Integer(cbBits.Items.Objects[cbBits.ItemIndex]));
  for i:= 0 to cbBits.Items.Count - 1 do
    if cbBits.Items.Objects[i] = TObject(aInt) then begin
      cbBits.ItemIndex:= i;
      Break;
    end;
  aInt:= aFile.ReadInteger(csSection, 'Sampling',
         Integer(cbSampling.Items.Objects[cbSampling.ItemIndex]));
  for i:= 0 to cbSampling.Items.Count - 1 do
    if cbSampling.Items.Objects[i] = TObject(aInt) then begin
      cbSampling.ItemIndex:= i;
      Break;
    end;
end;

procedure TfrWaveSetup.SaveIni(aFile: TIniFile);
begin
  aFile.WriteInteger(csSection, 'Input', cbDeviceIn.ItemIndex);
  aFile.WriteInteger(csSection, 'Output', cbDeviceOut.ItemIndex);
  aFile.WriteInteger(csSection, 'Bits',
                     Integer(cbBits.Items.Objects[cbBits.ItemIndex]));
  aFile.WriteInteger(csSection, 'Sampling',
                     Integer(cbSampling.Items.Objects[cbSampling.ItemIndex]));
end;

function TfrWaveSetup.Bits: Word;
begin
  Result:= Integer(cbBits.Items.Objects[cbBits.ItemIndex]);
end;

function TfrWaveSetup.Sampling: LongWord;
begin
  Result:= Integer(cbSampling.Items.Objects[cbSampling.ItemIndex]);
end;

function TfrWaveSetup.Input: Integer;
begin
  Result:= cbDeviceIn.ItemIndex;
end;

function TfrWaveSetup.Output: Integer;
begin
  Result:= cbDeviceOut.ItemIndex;
end;

procedure TfrWaveSetup.SetEnabled(Value: Boolean);
begin
  inherited SetEnabled(Value);
  cbBits.Enabled:= Value;
  cbSampling.Enabled:= Value;
  cbDeviceIn.Enabled:= Value;
  cbDeviceOut.Enabled:= Value;
end;

end.
