unit uSignalIO;

interface

uses
  Messages,
  uInterfaces, uWaveIO;

type
  TSignal = array of array of Double;
  TOnDataReady = procedure of object;

  TILO = record  // In, Local, Out
    I, L, O: Integer; 
  end;

  TSignalIn = class(TWaveIn)
    function GetAccMode: Integer; stdcall;
    procedure SetAccMode(aValue: Integer); stdcall;
    function GetAccCount: Integer; stdcall;
    procedure SetAccCount(aValue: Integer); stdcall;
  private
    FAccData: array of Int64;
    FAccPos: Integer;
    FAccCount: TILO;
    FAccMode: TILO;
    FCanSendF: Boolean;
    FReset: Boolean;
    FWinHD: Integer;
    FCount: Integer;
    procedure WndProc(var aMsg: TMessage);
    procedure SetCount(aValue: Integer);
    procedure SynchroProc;
  protected
    procedure SetChannels(aValue: Integer); override;
    procedure ThreadProc; override;
  public
    Data: TSignal;
    OnDataReady: TOnDataReady;
    constructor Create(aSetup: IWaveSetup);
    destructor Destroy; override;
    procedure Start; override;
    procedure CanSend;
    property Count: Integer read FCount write SetCount;
    property AccCount: Integer read GetAccCount write SetAccCount;
    property AccMode: Integer read GetAccMode write SetAccMode;
     // AccMode - accumulation mode
     // 0 - no accumulation, 1 - accumulation with autoreset
     // 2 - infinit accumulation, 3 - reset of infinit accumulation (write only)
  end;

  TSignalOut = class(TWaveOut)
  private
    FCount: Integer;  
  protected
    procedure SetChannels(aValue: Integer); override;  
  public
    constructor Create(aSetup: IWaveSetup);
    procedure SetCount(aValue: Integer);
  end;

implementation

uses
  Windows, Forms;

const
  WM_DATA = WM_USER + $000D;

constructor TSignalIn.Create(aSetup: IWaveSetup);
begin
  inherited Create(aSetup);
  FWinHD:= AllocateHWnd(WndProc);
  AccMode:= 0;
  FAccCount.I:= 4;
  OnDataReady:= nil;
  SetCount(1024);
end;

destructor TSignalIn.Destroy;
begin
  DeallocateHWnd(FWinHD);
  inherited;
end;

procedure TSignalIn.SetCount(aValue: Integer);
begin
  FCount:= aValue;
  SetLength(FAccData, Channels*FCount);
  SetLength(Data, Channels, FCount);
end;

procedure TSignalIn.SetChannels(aValue: Integer);
begin
  inherited SetChannels(aValue);
  SetCount(FCount);
end;

procedure TSignalIn.Start;
begin
  FCanSendF:= True;
  FAccPos:= 0;
  FAccCount.L:= 0;
  inherited Start;
end;

procedure TSignalIn.CanSend;
begin
  FCanSendF:= True;

end;

procedure TSignalIn.SynchroProc;
begin
  if FAccMode.I = 3 then begin
    FAccMode.I:= 2;
    FReset:= True;
  end else
    FReset:= False;
  FAccMode.L:= FAccMode.I;
end;

function TSignalIn.GetAccMode: Integer;
begin
  Result:= FAccMode.O;
end;

procedure TSignalIn.SetAccMode(aValue: Integer);
begin
  FAccMode.I:= aValue;
end;

function TSignalIn.GetAccCount: Integer;
begin
  Result:= FAccCount.O;
end;

procedure TSignalIn.SetAccCount(aValue: Integer);
begin
  FAccCount.I:= aValue;
end;

procedure TSignalIn.ThreadProc;
var
  aBNum, aANum: Integer;
  N, M, i: Integer;
  pB: ^LongInt;
  pA: ^Int64;
  pO: ^Double;
  aD: Extended;
begin
  Synchronize(SynchroProc);
  pB:= @FBufData[0];
  aBNum:= Length(FBufData);
  aANum:= Length(FAccData) - FAccPos;
  if (FAccMode.L + FAccMode.O = 0) and (aBNum > aANum) then begin
    N:= aBNum - aANum;
    M:= N div Length(FAccData);
    if M > 0 then begin
      N:= aANum + Pred(M) * Length(FAccData);
      Inc(pB, N);
      Dec(aBNum, N);
      FAccPos:= 0;
    end;
  end;
  while aBNum > 0 do begin
    if (FAccCount.L = 0) and (FAccPos = 0) then
      ZeroMemory(@FAccData[0], Length(FAccData)*SizeOf(FAccData[0]));
    aANum:= Length(FAccData) - FAccPos;
    pA:= @FAccData[FAccPos];
    if aBNum < aANum then
      N:= aBNum
    else
      N:= aANum;
    Inc(FAccPos, N);
    Dec(aBNum, N);
    Dec(aANum, N);
    while N > 0 do begin
      pA^:= pA^ + pB^;
      Inc(pA);
      Inc(pB);
      Dec(N);
    end;
    if (aANum = 0) then begin
      Inc(FAccCount.L);
      FAccPos:= 0;
      if ((aBNum < Length(FAccData)) and FCanSendF) and
      not ((FAccMode.L = 1) and (FAccCount.L < FAccCount.I)) then begin
        FCanSendF:= False;
        aD:= 1 / (FAccCount.L * (Int64(1) shl 31)); //    15
        M:= Channels;
        for i:= 0 to M-1 do begin
          pA:= @FAccData[i];
          pO:= @Data[i,0];
          N:= Length(Data[i]);
          while N > 0 do begin
            pO^:= pA^*aD;
            Inc(pA, M);
            Inc(pO);
            Dec(N);
          end;
        end;
        FAccCount.O:= FAccCount.L;
        if (FAccMode.L = 1) or FReset then
          FAccCount.L:= 0;
        FAccMode.O:= FAccMode.L;
        PostMessage(FWinHD, WM_DATA, 0, 0);
      end;
      if FAccMode.L = 0 then
        FAccCount.L:= 0;
    end;
  end;
end;

procedure TSignalIn.WndProc(var aMsg: TMessage);
begin
  if aMsg.Msg <> WM_DATA then
    Exit;
  if Assigned(OnDataReady) then
    OnDataReady;
end;

//------------------------------------------------------------------------------

constructor TSignalOut.Create(aSetup: IWaveSetup);
begin
  inherited Create(aSetup);
  SetCount(1000);
end;

procedure TSignalOut.SetChannels(aValue: Integer);
begin
  inherited SetChannels(aValue);
  SetCount(FCount);
end;

procedure TSignalOut.SetCount(aValue: Integer);
var
  i, j: Integer;
  p: ^LongInt;
begin
  FCount:= aValue;
  SetLength(FBufData, FCount*Channels);
  if FCount = 0 then
    Exit;
  p:= @(FBufData[0]);
  for i:= 0 to FCount-1 do
    for j:= 0 to Channels - 1 do begin
      p^:= LongInt(Round(0.99*High(LongInt)*Cos(2*Pi*(i+1)/FCount)));
      inc(p);
    end;
end;

end.
